'''
ikfkLeg.py by Malcolm Thomas-Gustave ( ikfk.com )
Built for 3d World Magazine
'''


#Imports for maya commands, access directly to mel, a little API with OpenMaya, and a python module with math
import maya.cmds as cmds
import maya.mel as mel
import maya.OpenMaya as om
import math

def locatorBuild(name=""):
	'''
	locatorBuild creates a default set of locators for our controls to work with
	name - Default name to use for the locators
	'''
	
	#Creates a base array to start out with
	locArray = []	
	
	#Array of positions to make locators
	locPos = [[0,5,0], [0,3,2.5], [0,1,0], [0,.5,2], [0,.5,4], [0,0,4], [0,0,-1]]
	
	#for in loop based on the length of locPos
	for i in range(0, len(locPos)):
		#if the current name for the locator doesn't exist in the scene it's created
		if not cmds.objExists((name+ "_Loc"+str(i+1))):
			#Appends the current locator into the array and moves it to the locPos positions
			locArray.append(cmds.spaceLocator(n=(name+ "_Loc"+str(i+1))))
			cmds.move(locPos[i][0], locPos[i][1], locPos[i][2], (name+ "_Loc"+str(i+1)))		
	
	#Returns locArray for further usage
	return locArray
	
def jointBuild(name="", jointOrient="xzy", secondAxisOrient="xup", locArray=[], end=5):
	'''
	jointBuild creates the joints along the postions of the locators
	name - base name for each joint
	jointOrient - jointOrient for each joint
	secondAxisOrient - Uses this value to determine how to do the jointOrient
	locArray - Array of all the locators to position along
	end - Where in the array to stop building

		NOTE: 
		For those that don't understand second axis orient in the jointOrient it's pretty simple.
		If you chose "xzy" as I have by default that means that "x" will be down the bone.
		But the system needs to know how you want to other two to angle themselves. Essentially the twist.
		In "xzy" you notice that "z" is the second axis. So secondAxisOrient is telling that axis where to aim.
		In this case it's trying to aim towards "xup" or positive X in the world.
		So the orienation twists around "x" until "z" is facing as close as it can to actual positive X in the world.
		This works well because the bones are along the yz plane.
		If the bones were different than this wouldn't work. Perhaps getting "z" to face "zup" or "yup" would work in those cases.
		Or possibly you want to change the orientation to "xyz" because you're more particular about your "y" than your "z".
		Play around with the tool and you'll find it's quite handy. 
		Just remember that jointOrient and rotationOrder are completely separate. 
		Some people get confused because of the option to change to different combinations.
	'''
	
	#Clears the selection, because otherwise the joints will be built underneath
	cmds.select(cl=True)
	
	#Again starting with a base array to have something to append into
	jnts=[]
	
	#For in loop starting from 0 to end so I don't have to go to the end of the array if i don't want to
	#This is useful when working with multiple arrays
	#In this case I used it for ease of name with the joints
	for i in range(0, end):
		#Queries the world space translate of each locator 
		trans = cmds.xform(locArray[i], q=True, t=True, ws=True)
		#Appends each joint in the array jnts
		jnts.append(cmds.joint(p=[trans[0],trans[1],trans[2]], n=(name+"_Jnt"+str(i))))
	
	#After the joint chain is constructed, the first one has a joint orient run on it using the function options
	#ch is True so that all the child joints are affect
	cmds.joint(jnts[0], e=True, oj=jointOrient, secondaryAxisOrient=secondAxisOrient, ch=True, zso=True)	
		
	#Returns the jnts array
	return jnts
		
def shapeAdd(array):
	'''
	shapeAdd is used to combine multiple curve objects together
	You pass it an array of curves and it combines all the shapes into the first curve
	'''
	
	#Freezes everything in the array to make sure the parenting will work correctly
	cmds.makeIdentity(array, apply=True)
	
	#for loop starting from 1 to the end of the array
	for i in range(1, len(array)):
		#Find the shape of each object in the array by using the listRelatives command set
		#for c(children) and s(shape)
		shapeChild = cmds.listRelatives(array[i], c=True, s=True, f=True)
		#Parent the shape into the first object in the array using the paren't s(shape) flag
		cmds.parent(shapeChild, array[0], s=True, add=True)
		#Delete each object in the array
		cmds.delete(array[i])
	
	#Deletes history on the first curve
	cmds.delete(array[0], ch=True)

def cylinderCurve(name="", scale=[1,1,1], colorVal=8):
	'''
	Makes a cylinder
	name - Name for the curve created
	scale - 3 object float array for the scale of the cylinder
	colorVal - color for the final curve
	'''
	
	#An array is created for curves and every curve is later added into this array
	curves = []

	baseCurve = cmds.circle(nr=[1,0,0])
	topCurve = cmds.circle(nr=[1,0,0], c=[3,0,0])

	#Circle returns an array so one way to work is with temp array values that you get the first index from
	curves.append(baseCurve[0])
	curves.append(topCurve[0])	
	
	#The curve command just returns a string so it's easy to append into our curves array
	curves.append( cmds.curve(d=3, p=[ (0,1,0), (1,1,0), (2,1,0), (3,1,0) ] ) )
	curves.append( cmds.curve(d=3, p=[ (0,0,0), (1,0,0), (2,0,0), (3,0,0) ] ) )
	curves.append( cmds.curve(d=3, p=[ (0,0,-1), (1,0,-1), (2,0,-1), (3,0,-1) ] ) )
	curves.append( cmds.curve(d=3, p=[ (0,0,1), (1,0,1), (2,0,1), (3,0,1) ] ) )
	curves.append( cmds.curve(d=3, p=[ (0,-1,0), (1,-1,0), (2,-1,0), (3,-1,0) ] ) )
	
	#This leaves us with just the baseCurve[0] object
	shapeAdd(curves)
	
	#The two scales are to show that you can manipulate things however you want
	#I like to get everything into a 0-1 range first and then scale
	#So by applying mulitple scales and then freezing you still get a 0-1
	cmds.scale(.33, .5, .5, curves[0])
	cmds.scale(scale[0], scale[1], scale[2], curves[0], r=True)
	cmds.makeIdentity(curves[0], apply=True)

	#The curve is renamed to name plus "Control"
	#I always catch renames back into a variable for use later
	#Technically you could cast back into curves[0] but for longer scripts it's sometimes easier to use new variables
	cylCurve = cmds.rename(curves[0], (name+"Control"))

	#The color gets changed using colorVal and the new cylCurve string
	cmds.color(cylCurve, ud=colorVal)

	#Final step is to return the cylCurve
	return(cylCurve)

def cubeCurve(name="", pos=[0,0,0], colorVal=6, scale=1):
	'''
	cubeCurve creates a curve shaped like a cube to use as a control
	name - Name for the curve created
	pos - Vector to move the control to
	colorVal - color for the final curve
	scale - Value to scale the curve up by
	'''
	
	#Another base array to store curves that I'm making and working with, followed by a shapeAdd
	curves = []
	curves.append( cmds.curve(d=1, p=[ (0,.5,.5), (0,.5,-.5), (0,-.5,-.5), (0,-.5,.5), (0,.5,.5)]) )
	curves.append( cmds.duplicate(curves[0])[0])
	cmds.move(.5,0,0, curves[1])	
	curves.append( cmds.duplicate(curves[0])[0])
	cmds.move(-.5,0,0, curves[2])	
	curves.append( cmds.duplicate(curves[0])[0])
	cmds.move(0,0,.5, curves[3])	
	cmds.rotate(0,90,0, curves[3])
	
	cmds.move(0,0,-.5, curves[0])
	cmds.rotate(0,90,0, curves[0])

	shapeAdd(curves)

	#Makes sure that the pivot is at zero, than moves the control to the pos vector
	cmds.move(0,0,0, (curves[0] + ".scalePivot"), (curves[0] + ".rotatePivot"))
	cmds.move(pos[0],pos[1],pos[2], curves[0])

	#Scales the control to whatever the scale value is
	cmds.scale(scale, scale, scale, curves[0], r=True)
	
	#Freezes the control
	cmds.makeIdentity(curves[0], apply=True)
	#Sets the color
	cmds.color(curves[0], ud=colorVal)
	#Renaming
	newName = cmds.rename(curves[0], (name+"Control"))
	
	#Return
	return(newName)
	
def baseControl(name=""):
	'''
	baseControl makes a control at the origin as a type of move all
	'''
	
	#Creates a curve and adds other curves, duplicating and rotating some to create the shapes.
	#Then the history gets deleted and the object is frozen
	base = cmds.circle(nr = [0,1,0])[0]
	first = cmds.curve(d=1,p=[ (1.093446, 0, -0.0928068), (1.470295, 0, -0.091131), (1.470922, 0, -0.232016), (1.725266, 0, 0.00169554), (1.468853, 0, 0.233136), (1.46948, 0, 0.0922507), (1.098473, 0, 0.0905749), (1.093446, 0, -0.0928068)])

	second = cmds.duplicate(first)[0]
	cmds.setAttr((second + ".ry"), 90)

	third = cmds.duplicate(first)[0]
	cmds.setAttr((third + ".ry"), 180)

	fourth = cmds.duplicate(first)[0]
	cmds.setAttr((fourth + ".ry"), 270)
	
	#This is the long form of what the shapeAdd does
	cmds.select(base, first, second, third, fourth)
	cmds.delete(ch=True)
	cmds.makeIdentity(apply=True)
	
	firstShape = cmds.listRelatives(first, c=True, s=True, f=True)
	secondShape = cmds.listRelatives(second, c=True, s=True, f=True)
	thirdShape = cmds.listRelatives(third, c=True, s=True, f=True)
	fourthShape = cmds.listRelatives(fourth, c=True, s=True, f=True)

	cmds.parent(firstShape, secondShape, thirdShape, fourthShape, base, s=True, add=True)
	cmds.delete(first, second, third, fourth)
	
	#Renaming with an "_BaseControl" and then a return
	baseName = cmds.rename(base, name+"_BaseControl")
	
	return baseName

def sphereCurve(name=""):
	'''
	sphereCurve just makes a sphere out of curves
	name - The default name
	'''
	
	#Creates all the curves, another long form of shapeAdd, renaming, and a return
	first = cmds.circle(nr = [0,1,0])[0]
	second = cmds.circle()[0]
	third = cmds.circle(nr = [1,0,0])[0]
	
	firstShape = cmds.listRelatives(first, c=True, s=True, f=True)
	secondShape = cmds.listRelatives(second, c=True, s=True, f=True)
	
	cmds.parent(firstShape, secondShape, third, s=True, add=True)
	cmds.delete(first, second)
	
	sphereCon = cmds.rename(third, name+"Control")
	
	return sphereCon
	
def ikfkBuild(name="", locBuild=1, locArray=[], fkConScale=2.5, ikConScale=2):
	'''
	ikfkBuild creates the whole structure for our leg
	name - Base name
	locBuild - 0 means don't run locatorBuild, 1 means run locator build
	locArray - Array of locators to query the position of for the script to know the proper positions
	fkConScale - Multiplier for the radius of the fkCons
	ikConScale - Multiplier for the scale of the ikCon
	'''
	#A little check to see if you input a locArray
	#If so then locBuild is turned off so another one isn't built
	if locArray:
		locBuild = 0
	
	#Because locBuild is 1 or 0 python reads that as true/false
	if locBuild:
		#If locBuild is true then it runs locatorBuild and locArray gets filled
		locArray = locatorBuild(name=name)

	#At this point locArray should exist, or it will error out 
	if not locArray:
		#Used the maya api to access an error message then returned
		om.MGlobal.displayError("ikfkBuild needs an array for locArray")
		return

	#Runs joint build on the locator array
	jnts = jointBuild(name=name, locArray=locArray, end=5)
	
	#Grabs the value of the 3 joint (the foot) and makes the ik control at that position
	trans = cmds.xform(jnts[2], q=True, ws=True, t=True)
	ikCon = cubeCurve(name+"_Ik", pos=trans, colorVal=6, scale=ikConScale)
	
	#Base grps for the FK controls and the groups over the FK controls
	fkGrps = []
	fkCons = []
	
	#For in loop to create the FK controls
	#It starts at 1 because I'll be skipping the last joint
	#I could have it stop at (len(jnts)-1) and start at 0 but I like throwing in different ways
	for i in range(1, len(jnts)):
		#Finds the scale by getting jnt[i]'s translate value
		#And gets the radius of jnt[i-1]
		#Because we're starting at 1 in the sequence I know there will always be one less that where we are
		scls = cmds.xform(jnts[i], q=True, t=True)
		rad = cmds.getAttr((jnts[i-1]+".radius"))
		#Creates a cylinder control. That translate value from the second joint is used as the X value
		#Because X is the length down the bone. 
		#The radius is input into the Y and Z scales times 1.25 because that gives it a little more thickness than the bone
		cylCon = cylinderCurve((name+"_Fk"+str(i)), scale=[scls[0], rad*fkConScale, rad*fkConScale], colorVal=8)
		#The control is grouped, the group's pivot goes to the origin
		grp = cmds.group(cylCon, n=(cylCon+"Grp"))
		cmds.xform(grp, os=True, piv=[0,0,0])
		#The group gets parent constrained to the proper joint because this is an easy way to get position and orientation. 
		#Then you just delete the constraint.
		pntConst = cmds.parentConstraint(jnts[i-1], grp)
		cmds.delete(pntConst)
		#The grp is appended to fkGrps and the control is appened to fkControls
		fkGrps.append(grp)
		fkCons.append(cylCon)
		
	#Now I'm using 0 to len(fkGrps)-1. Can give the same results as before, just gotta know how it works
	for i in range(0, len(fkGrps)-1):
		#Each grp is parented to the control that precedes it.
		#So the Knee group parents to the Hip control. The Foot group to the Knee control. ETC.
		cmds.parent(fkGrps[i+1], fkCons[i])
		
	#The jnts array is duplicated twice. Rename children is one to avoid naming conflicts
	#These two arrays will serve for the ikfk blend
	fkJnts = cmds.duplicate(jnts, renameChildren=True)
	ikJnts = cmds.duplicate(jnts, renameChildren=True)

	#All the joints are grouped together
	jntGrp = cmds.group(jnts[0], fkJnts[0], ikJnts[0], n=name+"_JntGrp")
	cmds.xform(jntGrp, os=True, piv=[0,0,0])

	#Here I'm using a for in loop to go through two array at the same time.
	#This makes it really easy to rename the whole thing in one go just the way we need.
	for i in range(0, len(fkJnts)):
		fkJnts[i] = cmds.rename(fkJnts[i], name+"_FkJnt"+str(i+1))
		ikJnts[i] = cmds.rename(ikJnts[i], name+"_IkJnt"+str(i+1))

	#The base control is created and the IK control and the main FK group gets parented in
	baseCon = baseControl(name=name)
	cmds.parent(ikCon, fkGrps[0], baseCon)

	#The base control gets the fkik attribute
	cmds.addAttr(baseCon, ln="fkIk", at="double", min=0, max=1, dv=1)
	cmds.setAttr(baseCon+".fkIk", e=True, keyable=True)

	#A pole vector control is created using the sphereCurve()
	poleVecCon = sphereCurve(name=name+"_PoleVec")

	#Again I highly recommend reading up on Vector math to really understand how this works.
	#This explains it all to a degree, but for a full understanding of how to expand on this you really need to do more study

	#World Space positions of the joints needed to position the pole vector
	jnt0WS = cmds.xform(jnts[0], q=True, ws=True, t=True)
	jnt1WS = cmds.xform(jnts[1], q=True, ws=True, t=True)
	jnt2WS = cmds.xform(jnts[2], q=True, ws=True, t=True)

	#Vector for the mid position between joint0 and joint2 in world space
	#1. Subtract jnt2WS from jnt0WS to get the vector from jnt2 to jnt0
	#2. Multiply that vector by .5 to get the mid position of that vector
	#3. Add that vector onto jnt2WS so that you can find the actual position in world space
	vecMid0to2 = [(((jnt0WS[0]-jnt2WS[0])*.5)+jnt2WS[0]), (((jnt0WS[1]-jnt2WS[1])*.5)+jnt2WS[1]),(((jnt0WS[2]-jnt2WS[2])*.5)+jnt2WS[2])]

	#Vector from between 0(hip) and 2(foot) to 1(the knee)
	vecMidTo1 = [(jnt1WS[0]-vecMid0to2[0]), (jnt1WS[1]-vecMid0to2[1]), (jnt1WS[2]-vecMid0to2[2])]
	
	#You get a vector's magnitude by multipling each element by itself, adding them together, then getting the square root of that final number
	#You can use the shorthand **2 for exponential multiplication by 2, which gives you an value multiplied by itself
	vecMidTo1Mag = math.sqrt((vecMidTo1[0]**2)+(vecMidTo1[1]**2)+(vecMidTo1[2]**2))

	#The normalized vector is derived by taking each vector element and dividing it by it's magnitude
	vecMidTo1Norm = [vecMidTo1[0]/vecMidTo1Mag, vecMidTo1[1]/vecMidTo1Mag, vecMidTo1[2]/vecMidTo1Mag]

	#The final Vector we're creating is for actually creating the world space position for where the Pole Vector should be
	#1. Multiply the normalized vector by 2 to create a 2 unit vector
	#2. Add this new value to jnt1 position so that you're offset from the knee by 2 units
	poleVecPos = [((vecMidTo1Norm[0]*2)+jnt1WS[0]),((vecMidTo1Norm[1]*2)+jnt1WS[1]), ((vecMidTo1Norm[2]*2)+jnt1WS[2])]

	#Makes a group and moves the pole vector control to the new position
	poleVecGrp = cmds.group(poleVecCon)
	cmds.xform(poleVecGrp, os=True, piv=[0,0,0])
	poleVecGrp = cmds.rename(poleVecGrp, poleVecCon+"Grp")
	cmds.move(poleVecPos[0], poleVecPos[1], poleVecPos[2], poleVecGrp)

	#Selects the start and end joint and makes the ikHandle
	cmds.select(ikJnts[0], ikJnts[2])
	ikHandle = cmds.ikHandle(n=name+"_Ik")
	
	#The pole vector control is made the pole vector for the leg ik
	cmds.poleVectorConstraint(poleVecCon, ikHandle[0])

	#Creates an array of the locators I need to make my footRoll
	footRollLocs = [locArray[-1], locArray[-2], locArray[-4], locArray[-5]]

	#Uses the jointBuild proc to create the footRoll joints
	footRollJnts = jointBuild(name=name+"footRoll", locArray=footRollLocs, end=4)

	#Groups the footRollJnts[0] for easy constraining later
	frGrp = cmds.group(footRollJnts[0])
	#Sets the pivot for the group to where the ikCon is located
	cmds.xform(os=True, piv=trans)
		
	#IKs for the appropriate parts of the footRoll
	cmds.select(ikJnts[2], ikJnts[3])
	ikHandle1 = cmds.ikHandle()
	cmds.select(ikJnts[3], ikJnts[4])
	ikHandle2 = cmds.ikHandle()

	#Parenting the Iks into the proper places. Also parenting the main IK into the structure
	cmds.parent(ikHandle[0], footRollJnts[3])
	cmds.parent(ikHandle1[0], footRollJnts[2])
	cmds.parent(ikHandle2[0], footRollJnts[1])

	#Adds the footRoll attribute to the ikCon
	cmds.addAttr(ikCon, ln="footRoll", at="double", min=-5, max=10, dv=0)
	cmds.setAttr(ikCon+".footRoll", e=True, keyable=True)

	#SDKs to make the footRoll work
	cmds.setDrivenKeyframe(footRollJnts[0]+".rz", cd=ikCon+".footRoll", dv=0, v=0)
	cmds.setDrivenKeyframe(footRollJnts[0]+".rz", cd=ikCon+".footRoll", dv=-5, v=-40)

	cmds.setDrivenKeyframe(footRollJnts[2]+".rz", cd=ikCon+".footRoll", dv=0, v=0)
	cmds.setDrivenKeyframe(footRollJnts[2]+".rz", cd=ikCon+".footRoll", dv=5, v=40)

	cmds.setDrivenKeyframe(footRollJnts[1]+".rz", cd=ikCon+".footRoll", dv=5, v=0)
	cmds.setDrivenKeyframe(footRollJnts[1]+".rz", cd=ikCon+".footRoll", dv=10, v=50)

	#A main leg group is made and the appropriate things are parented underneath
	legGrp = cmds.group( empty=True, n=name+"_Grp")
	cmds.xform(legGrp, os=True, piv=[0,0,0])
	cmds.parent(jntGrp, baseCon, poleVecGrp, frGrp, legGrp)

	#The IK control point and orient constrains the footRoll
	cmds.parentConstraint(ikCon, frGrp, mo=True)
	
	#Each normal joint is orient constrained to the corresponding ikJnt and fkJnt
	#SDKs are done for the orient constraints
	for i in range(0, (len(fkJnts)-1)):
		cmds.orientConstraint(fkCons[i], fkJnts[i], mo=True)
		orntConst = cmds.orientConstraint(ikJnts[i], fkJnts[i], jnts[i], mo=True)[0]
		cmds.setDrivenKeyframe(orntConst+".w0", cd=baseCon+".fkIk", dv=1, v=1)
		cmds.setDrivenKeyframe(orntConst+".w0", cd=baseCon+".fkIk", dv=0, v=0)
		cmds.setDrivenKeyframe(orntConst+".w1", cd=baseCon+".fkIk", dv=0, v=1)
		cmds.setDrivenKeyframe(orntConst+".w1", cd=baseCon+".fkIk", dv=1, v=0)
		
	#Constrains the jntGrp to make sure everything is moving together
	cmds.parentConstraint(baseCon, jntGrp)

def newSetupUI(*args):
	'''
	newSetup just makes a new scene and sets the layout to outliner 
	*args - There because of how I'm going to call this value later in the UI. Does nothing.
	'''
	#Forces the creation of a new scene
	cmds.file(new=True, f=True)
	#maya.mel as mel got imported at the beginning of the script
	#This gives access to all of the mel commands including eval
	#Using eval you can set the panel layout quite easily
	mel.eval('setNamedPanelLayout "Persp/Outliner";')

def locBuildUI(*args):
	'''
	locBuildUI uses the UI to build the locators
	*args - Because of the UI setup
	'''
	#Pulls the system name from the text field and runs locatorBuild on that name
	name = cmds.textField("ikfkLegFileTxtField",q=True, tx=True)
	locatorBuild(name)

def ikfkBuildUI(*args):
	'''
	ikfkBuildUI uses the UI for the name and the locators in the scene as the locArray
	*args - Still doin nothing, but has to be there
	'''

	#Pulls the system name from the text field
	#Creates a list of all the objects named with name and _Loc to create the locArray
	name = cmds.textField("ikfkLegFileTxtField",q=True, tx=True)
	locArray = cmds.ls(name+"_Loc*", type="transform")
	
	#The fkConScale and ikConScale are called by getting the string from the textField
	#Then using the float() command on the string value
	fkConScale = float(cmds.textField("ikfkLegFKTxtField", q=True, tx=True))
	ikConScale = float(cmds.textField("ikfkLegIKTxtField", q=True, tx=True))
	
	#Runs ikfkBuild on our name, locArray, fkConScale, and ikConScale
	ikfkBuild(name=name, locArray=locArray, fkConScale=fkConScale, ikConScale=ikConScale)

def deleteMainGrpUI(*args):
	'''
	deleteMainGrpUI deletes the main group
	*args - Now it's important. Just kidding.
	'''
	#Pulls the name, deletes the main grp using (name+"_Grp")
	name = cmds.textField("ikfkLegFileTxtField",q=True, tx=True)
	cmds.delete(name+"_Grp")
	
def ikfkWin():
	#If the window already exists than delete it
	if(cmds.window("ikfkLegWindow", ex=True)):
		cmds.deleteUI("ikfkLegWindow")
	
	#Creates a new window
	uiTitle = "IKFK Leg Setup"
	window = cmds.window("ikfkLegWindow", title=uiTitle , iconName='IKFKLeg', ret=False, s=True )
	
	#Main Layout
	cmds.columnLayout("ikfkLegUIMainColumn", adjustableColumn=True, p="ikfkLegWindow" )

	#Text Field with default text "defaultName"
	cmds.textField("ikfkLegFileTxtField",tx="defaultName", p="ikfkLegUIMainColumn")
	
	#Text and texfield for the IK con scale
	cmds.text(label="FK Controller Scale", p="ikfkLegUIMainColumn")
	cmds.textField("ikfkLegFKTxtField",tx="1.5", p="ikfkLegUIMainColumn")
	
	#Text and texfield for the FK con scale
	cmds.text(label="IK Controller Scale", p="ikfkLegUIMainColumn")
	cmds.textField("ikfkLegIKTxtField",tx="2", p="ikfkLegUIMainColumn")
	
	
	#Creates all the buttons
	cmds.button("ikfkLegNewSetupSceneBtn", label="New Setup (Creates New Scene)" , p="ikfkLegUIMainColumn", c=newSetupUI)
	cmds.button("ikfkLegLocatorBuildBtn", label="Locator Build" , p="ikfkLegUIMainColumn", c=locBuildUI)
	cmds.button("ikfkLegIKFKBuildBtn", label="IKFK Build" , p="ikfkLegUIMainColumn", c=ikfkBuildUI)
	cmds.button("ikfkLegDeleteMainGrpBtn", label="Delete Main Grp" , p="ikfkLegUIMainColumn", c=deleteMainGrpUI)

	#Shows the window and edits the size so it fits correctly
	cmds.showWindow( window )
	cmds.window(window, e=True, width=200, height=220)
ikfkWin()
'''
#README

#First off you need to import this module ( ikfkLeg ). So either place this script in a folder that's in your python script path or run the following.

import sys
sys.path.append(r"[pathtoyourscriptfolder]")

#Change [pathtoyourscriptfolder] with the full path.
#And yes leave the r there.
#It means raw so if you're working on a system with \ for your paths it won't read that as an escape character.

#This will allow you to now import ikfkLeg as thus

import ikfkLeg

#Once you have this module imported you have a couple things that you can do.

#1. Run the GUI
	ikfkLeg.ikfkWin()
#		From here you have a couple buttons and things you can do.
#			New Setup - This creates a new scene
#			Locator Build - This brings the locators in, you can move them around
#			IKFK Build - This builds the rig based on your locators
#			Delete Main Group - Does what it says, deletes the main group
#2. Run the scripts by themselves
	ikfkLeg.ikfkBuild(name="defaultName", locBuild=1, locArray=ikfkLeg.locatorBuild(name="defaultName"))
#	#What I did was run ikfkLeg.ikfkBuild with a couple options and then used ikfkLeg.locatorBuild to create the locArray
	
#	--- OR ---
	
	ikfkLeg.locatorBuild(name="defaultName")
		#Then move around the locators
	locArray = cmds.ls("defaultName_Loc*)
	ikfkLeg.ikfkBuild(name="defaultName", locBuild=1, locArray=locArray)
	
#If anything doesn't make sense, feel free to contact me through my website
'''
